@@ -1,6 +1,6 @@
 package Net::DNS::Packet;
 #
-# $Id: Packet.pm 546 2005-12-16 15:23:03Z olaf $
+# $Id: Packet.pm 579 2006-04-18 09:12:04Z olaf $
 #
 use strict;
 
@@ -22,7 +22,7 @@ require Exporter;
 @EXPORT_OK = qw(dn_expand);
 
 
-$VERSION = (qw$LastChangedRevision: 546 $)[1];
+$VERSION = (qw$LastChangedRevision: 579 $)[1];
 
 
 
@@ -711,7 +711,7 @@ sub dn_comp {
 		$self->{"compnames"}->{$dname} = $offset;
 		my $first  = shift @names;
 		my $length = length $first;
-		croak "length of $first is larger than 63 octets" if $length>63;
+		croak "length of $first is larger than 63 octets; see RFC1035 section 2.3.1" if $length>63;
 		$compname .= pack("C a*", $length, $first);
 		$offset   += $length + 1;
 	}
