@@ -1,6 +1,6 @@
 package Net::DNS::RR;
 #
-# $Id: RR.pm 552 2005-12-23 09:29:12Z olaf $
+# $Id: RR.pm 593 2006-05-25 09:40:39Z olaf $
 #
 use strict;
 
@@ -14,7 +14,7 @@ use Net::DNS;
 use Net::DNS::RR::Unknown;
 
 
-$VERSION = (qw$LastChangedRevision: 552 $)[1];
+$VERSION = (qw$LastChangedRevision: 593 $)[1];
 
 =head1 NAME
 
@@ -153,6 +153,22 @@ BEGIN {
 		    }
 		} 
 
+	 	eval { 
+		  local $SIG{'__DIE__'} = 'DEFAULT';
+		  require Net::DNS::RR::DLV; 
+		};
+
+		unless ($@) {
+		  $RR{'DLV'} =1;
+		} else {
+		  # Die only if we are dealing with a version for which DLV is 
+		  # available 
+		  die $@ if (
+			     defined ($Net::DNS::SEC::SVNVERSION) && 
+			     ( $Net::DNS::SEC::SVNVERSION > 591 )
+			    );
+		}
+
     }
 }
 
