@@ -1,26 +1,41 @@
 package Net::DNS::RR::MX;
 #
-# $Id: MX.pm,v 2.100 2003/12/13 01:37:05 ctriv Exp $
+# $Id: MX.pm 564 2006-02-20 09:34:26Z olaf $
 #
 use strict;
+BEGIN { 
+    eval { require bytes; }
+} 
 use vars qw(@ISA $VERSION);
 
-use Net::DNS;
-use Net::DNS::Packet;
-
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$Revision: 2.100 $)[1];
+$VERSION = (qw$LastChangedRevision: 564 $)[1];
+
+
+# Highest preference sorted first.
+__PACKAGE__->set_rrsort_func("preference",
+			       sub {
+				   my ($a,$b)=($Net::DNS::a,$Net::DNS::b);
+				   $a->{'preference'} <=> $b->{'preference'}
+}
+);
+
+
+__PACKAGE__->set_rrsort_func("default_sort",
+			       __PACKAGE__->get_rrsort_func("preference")
+
+    );
+
+
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
 
 	if ($self->{"rdlength"} > 0) {
-		my ($preference) = unpack("\@$offset n", $$data);
-		$offset += &Net::DNS::INT16SZ;
-		my ($exchange) = Net::DNS::Packet::dn_expand($data, $offset);
-
-		$self->{"preference"} = $preference;
-		$self->{"exchange"} = $exchange;
+		($self->{"preference"}) = unpack("\@$offset n", $$data);
+		$offset += Net::DNS::INT16SZ();
+		
+		($self->{"exchange"}) = Net::DNS::Packet::dn_expand($data, $offset);
 	}
 
 	return bless $self, $class;
@@ -41,9 +56,9 @@ sub new_from_string {
 sub rdatastr {
 	my $self = shift;
 
-	return exists $self->{"preference"}
+	return defined $self->{"preference"}
 	       ? "$self->{preference} $self->{exchange}."
-	       : "; no data";
+	       : '';
 }
 
 sub rr_rdata {
@@ -105,7 +120,8 @@ Returns name of this mail exchange.
 
 Copyright (c) 1997-2002 Michael Fuhr. 
 
-Portions Copyright (c) 2002-2003 Chris Reinhardt.
+Portions Copyright (c) 2002-2004 Chris Reinhardt.
+Portions Copyright (c) 2005 Olaf Kolkman NLnet Labs.
 
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
