@@ -1,13 +1,15 @@
 package Net::DNS::RR::NS;
 #
-# $Id: NS.pm 319 2005-05-30 17:12:09Z olaf $
+# $Id: NS.pm 554 2006-01-19 12:51:30Z olaf $
 #
 use strict;
-use bytes;
+BEGIN { 
+    eval { require bytes; }
+} 
 use vars qw(@ISA $VERSION);
 
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$LastChangedRevision: 319 $)[1];
+$VERSION = (qw$LastChangedRevision: 554 $)[1];
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
@@ -79,7 +81,7 @@ Class for DNS Name Server (NS) resource records.
 
     print "nsdname = ", $rr->nsdname, "\n";
 
-Returns the domain name of the nameserver.
+Returns the name of the nameserver.
 
 =head1 COPYRIGHT
 
@@ -87,6 +89,10 @@ Copyright (c) 1997-2002 Michael Fuhr.
 
 Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
+Portions Copyright (c) 2005 O.M, Kolkman, RIPE NCC.
+
+Portions Copyright (c) 2005-2006 O.M, Kolkman, NLnet Labs.
+
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
 
